/*
File Name: Bookmarks.java
Purpose: Sets up the bookmarks page, Contains function calls to the navigation drawer and to go to the Blog View activity
Related Files Used: activity_results.xml, content_results.xml, app_bar_results.xml, bookmarks.txt
Activities Accessed: Bookmarks.java, MainActivity.java, BlogView.java
Listed Functions: OnCreate, onBackPressed, doInBackground, onPostExecute, ProjectAdapter, getView, onNavigationItemSelected
Other Notes: Since both the Bookmarks and Results file are almost the same, with the exception of how data is accessed
    and processed, the Results display and template files are used, cutting down on the amount of files used by half
 */

package com.example.codys.compsciproject;

import android.content.Context;
import android.content.ContextWrapper;
import android.content.Intent;
import android.os.Bundle;
import android.support.design.widget.NavigationView;
import android.support.v4.view.GravityCompat;
import android.support.v4.widget.DrawerLayout;
import android.support.v7.app.ActionBarDrawerToggle;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.widget.Toolbar;
import android.view.LayoutInflater;

import android.os.AsyncTask;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.ListView;
import android.widget.TextView;
import android.widget.Toast;

import com.example.codys.compsciproject.models.ProjectModel;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

public class Bookmarks extends AppCompatActivity
        implements NavigationView.OnNavigationItemSelectedListener{

    private ListView ResultList;    // Contains the list from content_results.xml
    private Boolean empty_bm;       // Used as a flag, indicating if there are bookmarks or not

    /*
    Function Name: OnCreate
    Purpose: Initializes the activity.  Also, if the internal database does not exist, create it
    Function Type: Void
    Parameters Used: Bundle savedInstanceState (Refers to any data that was sent to this activity)
    Related Files: content_main.xml, activity_main.xml, app_bar_main.xml, nav_bar.xml, bookmarks.txt
    Other Notes:  The use of the result files is not a typo or error.  Please see the above
        for more information.
    */
    @Override
    protected void onCreate(Bundle savedInstanceState)
    {
        super.onCreate(savedInstanceState);
        // All we need to do is call activity_bookmarks, because it contains a call to content_main and app_bar_main as well
        setContentView(R.layout.activity_results);

        // Set up the toolbar at the top of the screen
        Toolbar toolbar = (Toolbar) findViewById(R.id.toolbar);
        setSupportActionBar(toolbar);

        // Grab the navigation drawer to apply open and close functions to it
        DrawerLayout drawer = (DrawerLayout) findViewById(R.id.drawer_layout);
        ActionBarDrawerToggle toggle = new ActionBarDrawerToggle(
                this, drawer, toolbar, R.string.navigation_drawer_open, R.string.navigation_drawer_close);

        // Sets up the navigation drawer to detect when the user taps on the hamburger (three bars in the top left corner)
        if (drawer != null)
            drawer.addDrawerListener(toggle);

        // Sets the item click events for the navigation drawer
        NavigationView navigationView = (NavigationView) findViewById(R.id.nav_view);
        if(navigationView != null)
            navigationView.setNavigationItemSelectedListener(this);

        // Allows the drawer to be opened and closed
        toggle.syncState();

        // List from content_results we will be storing the projects into
        ResultList = (ListView) findViewById(R.id.ProjectResults);

        // Set the bool indicating if there are bookmarks or not to false
        empty_bm = false;

        // Create the background thread to open and process the internal database
        new JSONTask().execute();
    }

    /*
    Function Name: onBackPressed
    Purpose: If the navigation drawer is open, the back button will allow it to be closed
    Function Type: Void
    Parameters Used: None
    Related Files: activity_main.xml, app_bar_main.xml, nav_bar.xml
    Other Notes: The use of the result files is not a typo or error.  Please see the
        above for more information.
    */
    @Override
    public void onBackPressed()
    {
        DrawerLayout drawer = (DrawerLayout) findViewById(R.id.drawer_layout);
        if (drawer != null) {
            drawer.closeDrawer(GravityCompat.START);
        } else {
            super.onBackPressed();
        }
    }

    /*
    Class Name: JSONTask
    Purpose: Contains the foreground and background threads for the database accessing, parsing, and list loading
    Related Files: content_blog_view.xml, activity_blog_view.xml, result_list.xml, database.txt (on CompSci02)
    */
    public class JSONTask extends AsyncTask<String, String, List<ProjectModel>>
    {
        /*
        Function Name: doInBackground
        Purpose: Runs before PostExecute, accesses the internal project database and sends it on to the foreground
        Function Type: List<ProjectModel>
        Parameters Used: String... params (Contains the parameters sent to it by the AsyncTask)
        Related Files: content_results.xml, activity_results.xml, result_list.xml
        Other Notes: The function type is List<ProjectModel> because it returns the filtered ProjectModel list to the
            foreground thread.  AsyncTask is the Android method used to establish a foreground and background thread.
            This utilizes a method similar to what is outlined in Results, but is ultimately different due to what kind
            of information is being accessed and what is being done with that information.
        */
        @Override
        protected List<ProjectModel> doInBackground(String... params)
        {
            try
            {
                String Result;  // Will be used to grab each line from the file
                String filepath = "MyFileStorage";  // Filepath to the internal database
                ContextWrapper contextWrapper = new ContextWrapper(getApplicationContext());  // Grabs the process the app is running in
                File directory = contextWrapper.getDir(filepath, Context.MODE_APPEND);  // Sets a file to the path to where the actual database is stored
                File file = new File(directory, "bookmarks.txt");   // Path to the actual database

                FileInputStream bm_file_in = new FileInputStream(file); // Stream to get information from the file
                InputStreamReader inputStreamReader = new InputStreamReader(bm_file_in);
                BufferedReader buff_read = new BufferedReader(inputStreamReader);
                StringBuilder buff_string = new StringBuilder();

                // While we can get information from the file, read it into the buffer.
                while ((Result = buff_read.readLine()) != null)
                {
                    buff_string.append(Result);
                    buff_string.append("\r\n");
                }

                bm_file_in.close();

                // Convert the buffer to a string.
                String finalJson = buff_string.toString();

                // As long as there are bookmarks, let's do something
                if (finalJson.trim().length() != 0)
                {
                    // Split up the string into an array based on the endline delimiter
                    List<String> bm_array = new ArrayList<>(Arrays.asList(finalJson.split("\\r?\\n")));
                    List<ProjectModel> FoundProjects = new ArrayList<>();   // List we will be sending on to the foreground thread
                    String current_line;        // Stores the string in the bm_array we are currently looking at
                    String[] current_array;     // Will be used to split up the information in current_line
                    ProjectModel current_project;   // ProjectModel object that will hold the information for the project we are looking at

                    for (int i = 0; i < bm_array.size(); i++) {
                        current_line = bm_array.get(i);  // Grabs all of the variables with each project entry
                        current_line = current_line.replace("{", "");   // Remove the left side bracket for the entry
                        current_line = current_line.replace("}", "");   // Remove the right side bracket for the entry
                        current_array = current_line.split(";");    // Split up the string into its individual values

                        // Load the project information into the ProjectModel
                        current_project = new ProjectModel();
                        current_project.setAuthor(current_array[2]);
                        current_project.setYear(Integer.parseInt(current_array[3]));
                        current_project.setTopic(current_array[0]);
                        current_project.setUrl(current_array[1]);
                        current_project.setKeywords("bookmarked");

                        // Add the current ProjectModel to the ProjectModel list
                        FoundProjects.add(current_project);
                    }

                    return FoundProjects;
                }
            }
            catch(IOException e)
            {
                e.printStackTrace();
            }

            // If bookmarks is empty, set the bool to true
            empty_bm = true;
            return null;
        }

        /*
        Function Name: onPostExecute
        Purpose: Runs after the thread parsing the project in Bookmarks is done.  Calls ProjectAdapter and
            loads the resulting item entries into the list in content_results.xml.  Also sets the onClick
            event for each of the items in the list.
        Function Type: Void
        Parameters Used: List<ProjectModel> result (Contains the parsed and filtered results from the background thread)
        Related Files: content_results.xml, activity_results.xml, result_list.xml
        Other Notes: This function runs when the background thread is done.  Also, the use of the result files is not
            a typo or error.  Please see the above for more information.
        */
        @Override
        protected void onPostExecute(final List<ProjectModel> result)
        {
            // If the user doesn't have any bookmarks saved, let them know
            if(empty_bm)
            {
                Toast.makeText(getApplicationContext(), "There are no bookmarks currently saved",
                        Toast.LENGTH_LONG).show();
                return;
            }

            super.onPostExecute(result);

            // Call ProjectAdapter and set up the list with the result_list.xml template and the project information
            ProjectAdapter adapter = new ProjectAdapter(getApplicationContext(), R.layout.result_list, result);
            ResultList.setAdapter(adapter);

            ResultList.setOnItemClickListener(new AdapterView.OnItemClickListener()
            {
                /*
                Function Name: onItemClick
                Purpose: Bundles the information for each project with its corresponding list item, and sets the call to BlogView
                    for each of the items
                Function Type: Void
                Parameters Used: AdapterView<?> parent (). View x (), int position, long id
                Related Files: content_results.xml, activity_results.xml, result_list.xml
                Other Notes: This function does run on every item in the list in content_results.xml.  Also, the use of the result
                files is not a typo or error.  Please see the above for more information.
                */
                @Override
                public void onItemClick(AdapterView<?> parent, View view, int position, long id)
                {
                    // Get the project at the listed position
                    ProjectModel thisProject = result.get(position);
                    Intent intent = new Intent(Bookmarks.this, BlogView.class);
                    Bundle BlogBund = new Bundle();

                    // Bundle all of the project information with the given list item
                    BlogBund.putString("LIST_PROJECT", thisProject.getTopic());
                    BlogBund.putString("LIST_URL", thisProject.getUrl());
                    BlogBund.putInt("LIST_YEAR", thisProject.getYear());
                    BlogBund.putString("LIST_NAME", thisProject.getAuthor());

                    // Send the project information on to BlogView
                    intent.putExtras(BlogBund);
                    startActivity(intent);
                }
            });
        }
    }

    /*
    Function Name: getView
    Purpose: Takes the results of the JSON parsing, loads it into the template defined in result_list, and loads
        a copy into each slot of the list in content_results.xml
    Function Type: View
    Parameters Used: int position (The index in the list in content_results.xml), view convertView
        (contains result_list.xml), ViewGroup parent (contains a reference to the main view for
        Results, and all sub references including the one to result_list.xml)
    Related Files: result_list.xml, activity_results.xml, content_results.xml
    Other Notes: While it contains no for loop, it executes on every item within AllProjects.  Also, the use of
    the result files is not a typo or error.  Please see the above for more information.
    */
    public class ProjectAdapter extends ArrayAdapter<ProjectModel>
    {
        public List<ProjectModel> AllProjects;
        private int resource;
        private LayoutInflater inflater;

        public ProjectAdapter(Context context, int resource, List<ProjectModel> objects)
        {
            super(context, resource, objects);
            AllProjects = objects;
            this.resource = resource;
            inflater = (LayoutInflater)getSystemService(LAYOUT_INFLATER_SERVICE);
        }

        /*
        Function Name: getView
        Purpose: Takes the results of the JSON parsing, loads it into the template defined in result_list, and loads
            a copy into each slot of the list in content_results.xml
        Function Type: View
        Parameters Used: int position (The index in the list in content_results.xml), view convertView
            (contains result_list.xml), ViewGroup parent (contains a reference to the main view for
            Results, and all sub references including the one to result_list.xml)
        Related Files: result_list.xml, activity_results.xml, content_results.xml
        Other Notes: While it contains no for loop, it executes on every item within AllProjects.  Also, the use of the
        result files is not a typo or error.  Please see the above for more information.
        */
        @Override
        public View getView(int position, View convertView, ViewGroup parent)
        {
            if(convertView == null)
                convertView = inflater.inflate(resource, null);

            // Text fields from result_list.xml
            TextView project_title;
            TextView student_name;
            TextView year_value;

            // Grab each of the fields in result_list.xml
            project_title = (TextView)convertView.findViewById(R.id.project_title);
            student_name = (TextView)convertView.findViewById(R.id.student_name);
            year_value = (TextView)convertView.findViewById(R.id.grad_year);

            // Take the information from the projects and load them into the fields in result_list.xml
            if(project_title != null)
                project_title.setText(AllProjects.get(position).getTopic());

            if(student_name != null)
                student_name.setText(AllProjects.get(position).getAuthor());

            if(year_value != null)
                year_value.setText(String.valueOf(AllProjects.get(position).getYear()));

            // Send the project's version of result_list back to onPostExecute
            return convertView;
        }
    }

    /*
    Function Name: onNavigationItemSelected
    Purpose: Sets up Navigation Drawer item click events for the current activity
    Function Type: Boolean
    Parameters Used: MenuItem item (Refers to the navigation drawer that item that was selected)
    Related Files: nav_bar.xml, activity_results.xml, app_bar_results.xml
    Other Notes: The "Statement with empty body" warning does not cause any problems.  Thus, the
        warning is suppressed using the SuppressWarnings override at the beginning.  Also, the use
        of the result files is not a typo or error.  Please see the above for more information.
    */
    @SuppressWarnings("StatementWithEmptyBody")
    @Override
    public boolean onNavigationItemSelected(MenuItem item)
    {
        // Gets the id of the menu item selected
        int id = item.getItemId();

        if (id == R.id.nav_search)
        {
            // If "Home" was selected, call the activity
            Intent search_page = new Intent(this, MainActivity.class);
            startActivity(search_page);
        }
        else if (id == R.id.nav_bookmarks)
        {
            // If "Bookmarks" was selected, call the activity
            Intent bookmark_page = new Intent(this, Bookmarks.class);
            startActivity(bookmark_page);
        }
        else if (id == R.id.nav_about)
        {
            // If "About" was selected, display the toast containing the app information
            Toast.makeText(getApplicationContext(), R.string.about_string,
                    Toast.LENGTH_LONG).show();
        }

        // Grab the drawer itself
        DrawerLayout drawer = (DrawerLayout) findViewById(R.id.drawer_layout);

        // As long as the drawer exists, close it
        if(drawer != null)
            drawer.closeDrawer(GravityCompat.START);

        return true;
    }
}

